@@ -13,9 +13,13 @@ from discord.ext.commands import Bot, errors
 from discord_slash import SlashCommand, context
 from expiringdict import ExpiringDict
 
-from spellbot.database import get_legacy_prefixes, initialize_connection
+from spellbot.database import (
+    db_session_manager,
+    get_legacy_prefixes,
+    initialize_connection,
+)
 from spellbot.errors import SpellbotAdminOnly, UserBannedError
-from spellbot.operations import safe_send_channel
+from spellbot.operations import safe_message_reply, safe_send_channel
 from spellbot.services.channels import ChannelsService
 from spellbot.services.guilds import GuildsService
 from spellbot.services.verifies import VerifiesService
@@ -55,9 +59,6 @@ class SpellBot(Bot):
         async with self.channel_locks[channel_xid]:  # type: ignore
             yield
 
-    async def on_ready(self) -> None:  # pragma: no cover
-        logger.debug("logged in as %s", self.user)
-
     async def create_spelltable_link(self) -> Optional[str]:
         if self.mock_games:
             return f"http://exmaple.com/game/{uuid4()}"
@@ -110,7 +111,7 @@ class SpellBot(Bot):
 
     async def on_message(self, message: discord.Message):
         if not message.guild or not hasattr(message.guild, "id"):
-            return await super().on_message(message)  # handle direct messages normally
+            return await super().on_message(message)  # handle DMs normally
         if (
             not hasattr(message.channel, "type")
             or message.channel.type != discord.ChannelType.text
@@ -118,27 +119,24 @@ class SpellBot(Bot):
             return  # ignore everything else, except messages in text channels...
         if message.flags.value & 64:
             return  # message is hidden, ignore it
-        await self.handle_verification(message)
+
+        async with db_session_manager():
+            await self.handle_verification(message)
+
         guild_xid = message.guild.id  # type: ignore
         if message.content.startswith(self.legacy_prefix_cache[guild_xid]):
-            try:
-                settings = Settings()
-                embed = discord.Embed()
-                embed.set_thumbnail(url=settings.ICO_URL)
-                embed.description = (
-                    "SpellBot uses slash commands now. Just type `/` to see the list of"
-                    " supported commands! It may take up to one hour for these commands"
-                    " to appear for the first time. Also note that SpellBot's"
-                    " invite link has changed. Your server admin may need to re-invite"
-                    " the bot using the [updated invite link](https://discordapp.com/api"
-                    "/oauth2/authorize?client_id=725510263251402832&permissions="
-                    "2416045137&scope=applications.commands%20bot) if slash commands do"
-                    " not show up after one hour."
-                )
-                embed.color = settings.EMBED_COLOR
-                await message.reply(embed=embed)
-            except Exception as ex:
-                logger.warning("warning: %s", ex, exc_info=True)
+            settings = Settings()
+            embed = discord.Embed()
+            embed.set_thumbnail(url=settings.ICO_URL)
+            embed.description = (
+                "SpellBot uses [slash commands]"
+                "(https://discord.com/blog/slash-commands-are-here) now."
+                " Type `/` to see the list of commands! If you don't see any, please"
+                " [re-invite the bot using its new invite link]"
+                f"({settings.BOT_INVITE_LINK})."
+            )
+            embed.color = settings.EMBED_COLOR
+            await safe_message_reply(message, embed=embed, delete_after=10.0)
 
     async def handle_verification(self, message: discord.Message):
         # To verify users we need their user id, so just give up if it's not available
@@ -149,8 +147,8 @@ class SpellBot(Bot):
         guilds = GuildsService()
         await guilds.upsert(message.guild)
         channels = ChannelsService()
-        await channels.upsert(message.channel)
-        if await channels.should_auto_verify():
+        channel_data = await channels.upsert(message.channel)
+        if channel_data["auto_verify"]:
             verified = True
         verify = VerifiesService()
         assert message.guild
@@ -158,9 +156,9 @@ class SpellBot(Bot):
         await verify.upsert(guild.id, message_author_xid, verified)
         if not user_can_moderate(message.author, guild, message.channel):
             user_is_verified = await verify.is_verified()
-            if user_is_verified and await channels.unverified_only():
+            if user_is_verified and channel_data["unverified_only"]:
                 await message.delete()
-            if not user_is_verified and await channels.verified_only():
+            if not user_is_verified and channel_data["verified_only"]:
                 await message.delete()
 
 
@@ -169,6 +167,7 @@ def build_bot(
     mock_games: bool = False,
     force_sync_commands: bool = False,
     clean_commands: bool = False,
+    create_connection: bool = True,
 ) -> SpellBot:
     bot = SpellBot(loop=loop, mock_games=mock_games)
 
@@ -184,6 +183,20 @@ def build_bot(
         delete_from_unused_guilds=clean_commands,
     )
 
+    # Note: In tests we create the connection using fixtures.
+    if create_connection:  # pragma: no cover
+
+        async def bot_connection(bot):
+            logger.info("initializing database connection...")
+            await initialize_connection("spellbot-bot")
+
+            async with db_session_manager():
+                logger.info("building legacy command prefix cache...")
+                db_legacy_prefixes = await get_legacy_prefixes()
+                bot.legacy_prefix_cache.update(db_legacy_prefixes)
+
+        bot.loop.run_until_complete(bot_connection(bot))
+
     # load all cog extensions
     from spellbot.cogs import load_all_cogs
 
@@ -191,15 +204,4 @@ def build_bot(
     commands = (key for key in bot.slash.commands if key != "context")
     logger.info("loaded commands: %s", ", ".join(commands))
 
-    # setup database connection
-    if not bot.loop.is_running():  # pragma: no cover
-        # In tests we will have to call initialize_connection manually due to
-        # pytest-asyncio having already started the async loop before we can get here.
-        logger.info("initializing database connection...")
-        bot.loop.run_until_complete(initialize_connection("spellbot-bot"))
-
-        logger.info("building legacy command prefix cache...")
-        db_legacy_prefixes = bot.loop.run_until_complete(get_legacy_prefixes())
-        bot.legacy_prefix_cache.update(db_legacy_prefixes)
-
     return bot
