@@ -10,6 +10,7 @@ from spellbot.utils import (
     DiscordChannel,
     bot_can_delete_channel,
     bot_can_read,
+    bot_can_reply_to,
     bot_can_role,
     log_warning,
     suppress,
@@ -110,14 +111,17 @@ async def safe_update_embed(message: discord.Message, *args, **kwargs) -> None:
         await message.edit(*args, **kwargs)
 
 
-async def safe_update_embed_origin(ctx: ComponentContext, *args, **kwargs) -> None:
+async def safe_update_embed_origin(ctx: ComponentContext, *args, **kwargs) -> bool:
     assert hasattr(ctx, "origin_message_id")
+    success = False
     with suppress(
         DiscordException,
         log="could not update origin embed in message %(message_xid)s",
         message_xid=ctx.origin_message_id,
     ):
         await ctx.edit_origin(*args, **kwargs)
+        success = True
+    return success
 
 
 async def safe_create_category_channel(
@@ -262,6 +266,15 @@ async def safe_send_channel(
     return message
 
 
+async def safe_message_reply(message: discord.Message, *args, **kwargs):
+    if not bot_can_reply_to(message):
+        return
+    try:
+        await message.reply(*args, **kwargs)
+    except Exception as ex:
+        logger.debug("debug: %s", ex, exc_info=True)
+
+
 async def safe_send_user(
     user: Union[discord.User, discord.Member],
     *args,
