@@ -1,4 +1,4 @@
-from datetime import datetime
+from datetime import datetime, timedelta
 from typing import Optional
 
 import discord
@@ -16,6 +16,7 @@ from spellbot.models.play import Play
 from spellbot.models.user import User
 from spellbot.models.watch import Watch
 from spellbot.services import BaseService
+from spellbot.settings import Settings
 
 MAX_SPELLTABLE_LINK_LEN = Game.spelltable_link.property.columns[  # type: ignore
     0
@@ -26,7 +27,8 @@ MAX_VOICE_INVITE_LINK_LEN = Game.voice_invite_link.property.columns[  # type: ig
 
 
 class GamesService(BaseService):
-    game: Optional[Game] = None
+    def __init__(self):
+        self.game: Optional[Game] = None
 
     @sync_to_async
     def select(self, game_id: int) -> bool:
@@ -45,7 +47,7 @@ class GamesService(BaseService):
         return bool(self.game)
 
     @sync_to_async
-    def select_by_message_xid(self, message_xid: int) -> bool:
+    def select_by_message_xid(self, message_xid: int) -> Optional[dict]:
         self.game = (
             DatabaseSession.query(Game)
             .filter(
@@ -53,16 +55,34 @@ class GamesService(BaseService):
             )
             .one_or_none()
         )
-        return bool(self.game)
+        return self.game.to_dict() if self.game else None
+
+    @sync_to_async
+    def current_status(self) -> bool:
+        assert self.game
+        rows = DatabaseSession.query(User).filter(User.game_id == self.game.id).count()
+        return rows + 1 <= self.game.seats
 
     @sync_to_async
     def add_player(self, player_xid: int) -> None:
         assert self.game
-        assert len(self.game.players) + 1 <= self.game.seats
+
+        rows = DatabaseSession.query(User).filter(User.game_id == self.game.id).count()
+        assert rows + 1 <= self.game.seats
+
         query = update(User).where(User.xid == player_xid).values(game_id=self.game.id)
         DatabaseSession.execute(query)
         DatabaseSession.commit()
 
+        # This operation should "dirty" the Game, so we need to update its updated_at.
+        query = (
+            update(Game)
+            .where(Game.id == self.game.id)
+            .values(updated_at=datetime.utcnow())
+        )
+        DatabaseSession.execute(query)
+        DatabaseSession.commit()
+
     @sync_to_async
     def upsert(
         self,
@@ -73,7 +93,54 @@ class GamesService(BaseService):
         friends: list[int],
         seats: int,
         format: int,
+        create_new: bool = False,
     ) -> bool:
+        existing: Optional[Game] = None
+        if not create_new:
+            existing = self._find_existing(
+                guild_xid=guild_xid,
+                channel_xid=channel_xid,
+                author_xid=author_xid,
+                friends=friends,
+                seats=seats,
+                format=format,
+            )
+
+        new: bool
+        game: Game
+        if existing:
+            game = existing
+            new = False
+        else:
+            game = Game(
+                guild_xid=guild_xid,
+                channel_xid=channel_xid,
+                seats=seats,
+                format=format,
+            )
+            DatabaseSession.add(game)
+            DatabaseSession.commit()
+            new = True
+
+        DatabaseSession.execute(
+            update(User)
+            .where(User.xid.in_([*friends, author_xid]))
+            .values(game_id=game.id)
+        )
+        DatabaseSession.commit()
+        self.game = game
+        return new
+
+    def _find_existing(
+        self,
+        *,
+        guild_xid: int,
+        channel_xid: int,
+        author_xid: int,
+        friends: list[int],
+        seats: int,
+        format: int,
+    ):
         required_seats = 1 + len(friends)
         inner = (
             select(
@@ -120,32 +187,7 @@ class GamesService(BaseService):
                 )
             )
         )
-        existing = outer.first()
-
-        new: bool
-        game: Game
-        if existing:
-            game = existing
-            new = False
-        else:
-            game = Game(
-                guild_xid=guild_xid,
-                channel_xid=channel_xid,
-                seats=seats,
-                format=format,
-            )
-            DatabaseSession.add(game)
-            DatabaseSession.commit()
-            new = True
-
-        DatabaseSession.execute(
-            update(User)
-            .where(User.xid.in_([*friends, author_xid]))
-            .values(game_id=game.id)
-        )
-        DatabaseSession.commit()
-        self.game = game
-        return new
+        return outer.first()
 
     @sync_to_async
     def to_embed(self, dm: bool = False) -> discord.Embed:
@@ -171,15 +213,15 @@ class GamesService(BaseService):
         return self.game.channel_xid
 
     @sync_to_async
-    def current_message_xid(self) -> int:
+    def current_message_xid(self) -> Optional[int]:
         assert self.game
-        assert self.game.message_xid
         return self.game.message_xid
 
     @sync_to_async
     def fully_seated(self) -> bool:
         assert self.game
-        return self.game.seats == len(self.game.players)
+        rows = DatabaseSession.query(User).filter(User.game_id == self.game.id).count()
+        return rows == self.game.seats
 
     @sync_to_async
     def make_ready(self, spelltable_link: Optional[str]) -> None:
@@ -193,7 +235,8 @@ class GamesService(BaseService):
     @sync_to_async
     def current_player_xids(self) -> list[int]:
         assert self.game
-        return [player.xid for player in self.game.players]
+        rows = DatabaseSession.query(User.xid).filter(User.game_id == self.game.id)
+        return [int(row[0]) for row in rows]
 
     @sync_to_async
     def watch_notes(self, player_xids: list[int]) -> dict[int, Optional[str]]:
@@ -215,7 +258,9 @@ class GamesService(BaseService):
         assert self.game
         assert self.game.status == GameStatus.STARTED.value
 
-        player_xids = [player.xid for player in self.game.players]
+        rows = DatabaseSession.query(User.xid).filter(User.game_id == self.game.id)
+        player_xids = [int(row[0]) for row in rows]
+
         game_id = self.game.id
         guild_xid = self.game.guild_xid
 
@@ -282,7 +327,9 @@ class GamesService(BaseService):
     @sync_to_async
     def blocked(self, author_xid: int) -> bool:
         assert self.game
-        other_player_xids = [player.xid for player in self.game.players]
+        rows = DatabaseSession.query(User.xid).filter(User.game_id == self.game.id)
+        other_player_xids = [int(row[0]) for row in rows]
+
         query = DatabaseSession.query(Block).filter(
             or_(
                 and_(
@@ -302,6 +349,11 @@ class GamesService(BaseService):
         assert self.game
         return self.game.jump_link
 
+    @sync_to_async
+    def to_dict(self) -> dict:
+        assert self.game
+        return self.game.to_dict()
+
     @sync_to_async
     def players_included(self, player_xid: int) -> bool:
         """
@@ -341,3 +393,21 @@ class GamesService(BaseService):
         )
         DatabaseSession.execute(upsert, values)
         DatabaseSession.commit()
+
+    @sync_to_async
+    def inactive_games(self) -> list[dict]:
+        settings = Settings()
+        limit = datetime.utcnow() - timedelta(minutes=settings.EXPIRE_TIME_M)
+        records = DatabaseSession.query(Game).filter(
+            and_(
+                Game.status == GameStatus.PENDING.value,
+                Game.updated_at <= limit,
+            )
+        )
+        return [record.to_dict() for record in records]
+
+    @sync_to_async
+    def delete_games(self, game_ids: list[int]):
+        query = DatabaseSession.query(Game).filter(Game.id.in_(game_ids))
+        query.delete(synchronize_session=False)
+        DatabaseSession.commit()
