@@ -25,6 +25,21 @@ def log_warning(log: str, exec_info: bool = False, **kwargs: Any):
     logger.warning(message, kwargs, exc_info=exec_info)
 
 
+def bot_can_reply_to(message: discord.Message) -> bool:
+    if (
+        not hasattr(message, "channel")
+        or not hasattr(message.channel, "type")
+        or not hasattr(message.channel, "guild")
+        or message.channel.type != discord.ChannelType.text
+    ):
+        return False
+    perms = message.channel.permissions_for(message.channel.guild.me)
+    for req in ("send_messages",):
+        if not hasattr(perms, req) or not getattr(perms, req):
+            return False
+    return True
+
+
 def bot_can_role(guild: discord.Guild) -> bool:
     if not guild.me:
         return False
@@ -103,10 +118,10 @@ def user_can_moderate(
     author_id = author.id  # type: ignore
     if author_id == guild.owner_id:
         return True
-    if (perms := channel.permissions_for(author)) and perms.administrator:
-        return True
     if not hasattr(author, "roles"):
         return False
+    if (perms := channel.permissions_for(author)) and perms.administrator:
+        return True
     author_roles = author.roles  # type: ignore
     settings = Settings()
     return any(
